%=======================================================================
function Pht
%=======================================================================
% loads a mat file generated via load_edis, containing impedances and
% calculates and plots parameters of Caldwell's Phase tensor ellipses
% -> very much analogous to mj_pcolor.m
%
% ws, april '04
%=======================================================================
global ph_svd parplot
% preferred strike direction -> for projection along line
pref_strike = -30*pi/180;
azimuth = 0;  nsect=20; dcrit=5;

% load impedance array
[matfile,matpath] = uigetfile({'*.mat'},'pick mat file');
matfile = fullfile(matpath,matfile);
eval(['load ',matfile]);

% load impedance array
%[matfile,matpath] = uigetfile({'*.mat'},'pick mat file');
%matfile = fullfile(matpath,matfile);
%eval(['load ',matfile]);

per = T; logper = log10(per);
nper = length(per)
nsites = size(coord,1)
   
% calculate phase tensors a la Caldwell
for isite=1:nsites
    z = squeeze(ZZZ(:,:,isite));
    dz = squeeze(EEE(:,:,isite));
    [pht,phsvd,dpht,dphsvd] = ph_tensor_edi(z,dz,per);
    % complete array:
    ph_tens(:,:,isite)  = pht;
    ph_svd(:,:,isite)   = phsvd;
    dph_tens(:,:,isite) = dpht;
    dph_svd(:,:,isite)  = dphsvd;
    % here mostly interested in ph_svd: ph_max, ph_min, alpha, beta
end

% why not also look at Swift's skew
swift_skew = squeeze(abs(ZZZ(1,:,:)+ZZZ(4,:,:))./abs(ZZZ(2,:,:)-ZZZ(3,:,:)));                
% error... too lazy for now.
dswift_skew = zeros(nper,nsites);

% and Bahr skew
s1 = ZZZ(1,:,:)+ZZZ(4,:,:);  s2 = ZZZ(2,:,:)+ZZZ(3,:,:); 
d1 = ZZZ(1,:,:)-ZZZ(4,:,:);  d2 = ZZZ(2,:,:)-ZZZ(3,:,:);
c  = (real(d1).*imag(s2)-imag(d1).*real(s2)) - ...
     (real(s1).*imag(d2)-imag(s1).*real(d2));
bahr_skew = squeeze(sqrt(abs(c))./abs(d2));

% and Bahr strike - just for test against alpha
A  = -((real(d1).*imag(s1)-imag(d1).*real(s1))  +  (real(d2).*imag(s2)-imag(d2).*real(s2)));
B  =   (real(s1).*imag(s2)-imag(s1).*real(s2))  -  (real(d1).*imag(d2)-imag(d1).*real(d2));
%alpha = squeeze(atan(B./A)/2 *180/pi);

% errors - see above
dbahr_skew = zeros(nper,nsites);

tph_max = squeeze(ph_svd(1,:,:));
tph_min = squeeze(ph_svd(2,:,:));
ph_max  = atan(squeeze(ph_svd(1,:,:)))*180/pi;
ph_min  = atan(squeeze(ph_svd(2,:,:)))*180/pi;
alpha   = squeeze(ph_svd(3,:,:))*180/pi;
beta    = squeeze(ph_svd(4,:,:))*180/pi;
% map [-45,45] into [0,90] (don't forget to adjust caxis!)
alpha90 = rem(rem(alpha,360)+360,90);
% interval: -45;45
alpha45 = alpha90;
lindex = find(alpha45>45);
alpha45(lindex)=alpha45(lindex)-90;
dph_max=squeeze(dph_svd(1,:,:));
dph_min=squeeze(dph_svd(2,:,:));
dalpha =squeeze(dph_svd(3,:,:))*180/pi;
dbeta  =squeeze(dph_svd(4,:,:))*180/pi;

% indices:      1      2      3    4       5           6         7
param = cat(3,ph_max,ph_min,alpha45,beta,alpha45-beta,swift_skew,bahr_skew);
param = permute(param,[3 2 1]);
% -> parameter, site, period
dparam = cat(3,dph_max,dph_min,dalpha,dbeta,dalpha,dswift_skew,dbahr_skew);
dparam = permute(dparam,[3 2 1]);

lat  = coord(:,2)';
long = coord(:,1)';
cent_lat  = (max(lat)+min(lat))/2;
cent_long = (max(long)+min(long))/2;

% Convert degrees to km N-S and E-W
x = 111.*cos(cent_lat*pi/180)*(long-cent_long); 
y = 111.*(lat-cent_lat);
% project stations on line of preferred strike (sp)
beta = atan2(y,x); 
r = sqrt(x.^2 + y.^2);
xdist = r.*cos(pref_strike+beta);

%===========================================
% Plot
%===========================================
choice1 = 1;
while choice1 ~= 6
    menu1 = {'pcolor','map','profile','station','rose','exit'};
    menu2 = {'phi-max','phi-min','alpha','beta','alpha-beta',...
             'swift-skew','bahr-skew','phase-ellipse','induction-vectors'};

    choice1 = menu(' kind of plot ',menu1);
    if choice1 ~= 6
        choice2 = menu(' parameter ',menu2);
    else
        % dummy -> exit.
        choice2 = 1;
    end
    if choice2 < 8 & choice1 <6
        parplot = squeeze(param(choice2,:,:));
        dparplot = squeeze(dparam(choice2,:,:));
    end
    switch choice2
        case 1  % ph_max
            pclims=[0,90];
            pylabel='degrees';
            cmap=jet(32);
        case 2 % ph_min
            pclims=[0 90];
            pylabel='degrees';
            cmap=jet(32);
        case 3 % alpha
            pclims=[-45 45];
            pylabel='degrees';
            cmap=hsv(32);
        case 4 % beta
            pclims=[-10 10];
            pylabel='degrees';
            cmap=jet(32);
        case 5 % alpha-beta
            pclims=[-45 45];
            pylabel='degrees';
            cmap=hsv(32);
        case 6 % swift_skew
            pclims=[0 .6];
            pylabel=' ';
            cmap=jet(32);
        case 7 % bahr_skew
            pclims=[0 .6];
            pylabel=' ';
            cmap=jet(32);
    end
    switch choice1
        case 1
            if choice2==8 break; end
            % =================> PCOLOR
            figure
            set(gcf,'Name',[pwd,' - pcolor']);
            % adjust blocks such that sites are in their middle:
            % if spacing doesn't seem to be appropriate, check
            % dx for negative entries and reorder sitefile.
            dx=diff(xdist);
            % shift everything half dx to the left
            tx(1)=xdist(1)-dx(1)/2;
            tx(2:nsites)=xdist(1:nsites-1)+dx/2;
            tx(nsites+1)=xdist(nsites)+dx(2);
            % extend arrays
            parplot(nsites+1,:)=parplot(nsites,:);
            parplot(:,nper+1)=parplot(:,nper);
            per(nper+1)=per(nper);
            % plot
            %pcolor(parplot')
            pcolor(tx,log10(per),parplot');
            hold on
            plot(xdist,per(1)*ones(nsites,1),'kv');
            %plot(xdist,log10(per(1))*ones(nsites,1),'kv');
            set(gca,'YDir','reverse');
            shading flat
            colormap(cmap)
            caxis(pclims);
            colorbar('vert')
            xlabel('km'); ylabel('log period'); 
            title(['ABC profile: ',char(menu2(choice2))]);
            set(gcf, 'PaperPositionMode', 'auto'); 
            print('-dpsc',['abc-',char(menu2(choice2)),'-pcolor.ps']);
        case {2,3,4,5} % all others
	 	    prompt = {'choose period(s)'};
            titles  = 'Choose period (range)';
            def = {[num2str(nper),':',num2str(nper)]};
            ipers = char(inputdlg(prompt,titles,1,def))
            tper = per(str2num(ipers))
            if (choice1==2 & (choice2==3 | choice2==5 | choice2==8 | choice2==9))
                % =================> MAP
                figure
                plot(x,y,'k.','MarkerSize',15)
                hold on
                load map_matlab.dat
                load map_matlab_ind.dat
                begs=map_matlab_ind(1:length(map_matlab_ind)-1);
                ends=map_matlab_ind(2:length(map_matlab_ind))-1;
                map_matlab(:,1) = 111.*cos(cent_lat*pi/180)*(map_matlab(:,1)-cent_long);
                map_matlab(:,2) = 111.*(map_matlab(:,2)-cent_lat);
                for iline=1:length(begs)
                    line(map_matlab(begs(iline):ends(iline),1),map_matlab(begs(iline):ends(iline),2),...
                        'Color',[.6 .6 .6],'LineWidth',.5);
                end
                if (choice2==3 | choice2==5)
                    % ==> strike bars
                    parplot=squeeze(mean(parplot(:,str2num(ipers)),2)); 
                    parplot=parplot*pi/180;
                    % rotate to align strikes to a preferred direction
                    strike=parplot;
                    while 1
                        nchange=0;
                        aindex=find(strike-pref_strike >= pi/4);
                        bindex=find(strike-pref_strike < -pi/4);
                        if (sum(aindex)>0) strike(aindex)=strike(aindex)-pi/2; nchange=1; end
                        if (sum(bindex)>0) strike(bindex)=strike(bindex)+pi/2; nchange=1; end
                        if nchange==0 break; end
                    end
                    % create and plot bars
                    strike_scale=10;
                    x_strike=strike_scale.*cos(strike'); 
                    y_strike=strike_scale.*sin(strike');
                    xs = [x-x_strike; x+x_strike]; 
                    ys = [y+y_strike; y-y_strike];
                    xs_pi2 = [x-y_strike; x+y_strike]; 
                    ys_pi2 = [y-x_strike; y+x_strike];
                    plot(xs,ys,'b-')
                    plot(xs_pi2,ys_pi2,'r-') 
                    ctitle = [', preferred strike: ',num2str(pref_strike*180/pi),' deg'];
                elseif choice2==8
                    for isite=1:nsites
                        ell_scale = 11;
                        a = ell_scale.*mean(squeeze(tph_max(str2num(ipers),isite)));
                        b = ell_scale.*squeeze(sum(tph_min(str2num(ipers),isite)));
                        %b = 0;
                        theta = mean(squeeze(alpha(str2num(ipers),isite))).*pi/180;
                        theta = pi/2-theta;
                        % note: theta(cart)=90-theta(geo)
                        [x_ell,y_ell] = ellipse(a,b,theta,x(isite),y(isite),100);
                        plot(x_ell,y_ell);
                    end
                    % 30, 45, 60 deg circles
                    norm_ang = [25 45 65];
                    norm_ell = tan(norm_ang*pi/180)*ell_scale;
                    for inorm=1:3
                        [x_ell,y_ell] = ellipse(norm_ell(inorm),norm_ell(inorm),0,-150+(inorm-1)*50,100,100); 
                        plot(x_ell,y_ell);
                    end
                    ctitle = '';    
                elseif choice2==9
                    for isite=1:nsites
                        iv_scale=100;
                        locs=[x(isite) y(isite)];
                        % E = x; N = y
                        arr_end(1)=mean(squeeze(ZZZ_HZ(2,str2num(ipers),isite)))*iv_scale;
                        arr_end(2)=mean(squeeze(ZZZ_HZ(1,str2num(ipers),isite)))*iv_scale;
                        %arr_len=sqrt(arr_end(1)^2+arr_end(2)^2);
                        %rearr = arrow(locs,locs+real(arr_end),6*arr_len/50,90,10,.15);
                        rearr = arrow(locs,locs+real(arr_end),5,60,10,.2);
                        set(rearr,'FaceColor',[1 0 0],'EdgeColor',[1 0 0]);
                        %imarr = arrow(locs,locs+imag(arr_end),6*arr_len/50,90,10,.15);
                        imarr = arrow(locs,locs+imag(arr_end),5,60,10,.2);
                        set(imarr,'FaceColor',[0 0 1],'EdgeColor',[0 0 1]);
                    end
                    arrow([150 -100],[150 -100]+iv_scale.*[0 1],5,60,10,.2);
                    ctitle = '';    
                end
                axis('equal');
                axis([min(x)-30,max(x)+30,min(y)-30,max(y)+30]);
                xlabel('km'); ylabel('km');
                title([' ABC profile, ',char(menu2(choice2)),' period band: ',num2str(tper(1)),...
                       's-',num2str(tper(length(tper))),'s;',ctitle]);
                print('-dpsc',['abc-',char(menu2(choice2)),'-map.ps']);
elseif choice1==3                   
                % =================> PROFILE
                parplot=squeeze(mean(parplot(:,str2num(ipers)),2));
                figure
                plot(xdist,parplot,'+');
                xlabel(['km along preferred strike: ',num2str(pref_strike*180/pi)]);
                ylabel(pylabel);
                title(['abc profile;',...
                       ' period band: ',num2str(tper(1)),'s-',num2str(tper(length(tper))),'s;',...
                       ' ',char(menu2{choice2})]);
                set(gca,'XLim',[min(xdist)-10;max(xdist)+10],'YLim',pclims);
                print('-dpsc',['abc-',char(menu2(choice2)),'-profile.ps']);
            elseif (choice1==4)
                % =================> STATION
                % -> still to be implemented
            elseif (choice1==5 & (choice2==3 | choice2==5)) 
                % =================> ROSE
                parplot=squeeze(parplot(:,str2num(ipers)));
                % specify site range
	 	        prompt = {'choose site range'};
                stitle  = 'Choose site range';
                def = {['1:',num2str(nsites)]};
                isites = char(inputdlg(prompt,stitle,1,def));
                parplot = parplot(str2num(isites),:)*pi/180;
                %plot
                figure
                parplot(size(parplot,1)+1:2*size(parplot,1),:)=parplot+pi;
                parplot=reshape(parplot,1,[]);
                size(parplot)
                rose(parplot+pi/2,nsect);
                set(findobj(gca,'Type','line'),'Color','r');
                hold on
                rose(parplot,nsect);
                title(char(menu2{choice2}));
                print('-dpsc',['abc-',char(menu2(choice2)),'-rose.ps']);
            end
    end
end