%==========================================================================
function [ph_tens,ph_svd,dph_tens,dph_svd] = ...
          ph_tensor_edi(z,dz,periods)
%==========================================================================
%
% calculates phase tensor and ~SVD according to Caldwell & al. in GJI 2003
% ws april '04
%
%==========================================================================

nbt = length(periods);
nt  = size(z,2);

% real and imaginary parts of z
x = real(z);
y = imag(z);

det_x = x(1,:).*x(4,:) - x(2,:).*x(3,:);
ph11 = (x(4,:).*y(1,:) - x(2,:).*y(3,:))./det_x;
ph12 = (x(4,:).*y(2,:) - x(2,:).*y(4,:))./det_x; 
ph21 = (x(1,:).*y(3,:) - x(3,:).*y(1,:))./det_x;
ph22 = (x(1,:).*y(4,:) - x(3,:).*y(2,:))./det_x;
 
% below: see bibby '77 in geophysics vol 42.
phi_1 = 0.5 .* sqrt((ph11-ph22).^2 + (ph12+ph21).^2);
phi_2 = 0.5 .* sqrt((ph11+ph22).^2 + (ph12-ph21).^2);

phmax = phi_1+phi_2;
phmin = phi_2-phi_1;

alpha = atan((ph12+ph21)./(ph11-ph22))./2;
beta  = atan((ph12-ph21)./(ph11+ph22))./2;

% to check and learn a bit more about it
for ii=1:nt
    % check eq. 10 in bibby '86
    %p1 = 0.5 .* sqrt((ph11(ii)-ph22(ii)).^2 + (ph12(ii)+ph21(ii)).^2);
    %p2 = 0.5 .* sqrt((ph11(ii)+ph22(ii)).^2 + (ph12(ii)-ph21(ii)).^2);
    %mcs1 = [cos(2*alpha(ii)) sin(2*alpha(ii)); sin(2*alpha(ii)) -cos(2*alpha(ii))];
    %mcs2 = [cos(2*beta(ii)) sin(2*beta(ii)); -sin(2*beta(ii)) cos(2*beta(ii))];

    % below: the middle 'v' is indeed [phmax 0; 0 phmin]!
    %if ~isnan(ph11(ii))
    %    [u,v,w]=svd([ph11(ii) ph12(ii); ph21(ii) ph22(ii)]);
    %end
    cm = cos(alpha(ii)-beta(ii)); sm = sin(alpha(ii)-beta(ii));
    cp = cos(alpha(ii)+beta(ii)); sp = sin(alpha(ii)+beta(ii));
    rm = [cm -sm; sm cm]; rp = [cp -sp; sp cp];
    %cm90 = cos(alpha(ii)+pi/2-beta(ii)); sm90 = sin(alpha(ii)+pi/2-beta(ii));
    %cp90 = cos(alpha(ii)+pi/2+beta(ii)); sp90 = sin(alpha(ii)+pi/2+beta(ii));
    %rm90 = [cm90 -sm90; sm90 cm90]; rp90 = [cp90 -sp90; sp90 cp90];
    
    % below: pha, phb, phc are all the same!
    
    pha=[ph11(ii) ph12(ii); ph21(ii) ph22(ii)];
    phb=rm*[phmax(ii) 0; 0 phmin(ii)]*rp';
    
    %phb90=rm90*[phmax(ii) 0; 0 phmin(ii)]*rp90'
    %r90=[cos(pi/2) -sin(pi/2); sin(pi/2) cos(pi/2)];
    %phb90=r90*phb*r90'
    %phc=p1*mcs1+p2*mcs2

    if abs(phb(1,1)-pha(1,1)) < 1E-5
        alpha(ii)=alpha(ii)+pi/2; 
    end
    % note that if Ph(Ex/Hy)>Ph(Ey/Hx)+180, Ph11<Ph22 -> not quite intuitive!
end

% note: here and in caldwell's paper, tensor and entries are called
% phi, but actually it's y/x, or im/re, so the tangens of the phase!

%==========================================================================
% compute error via gaussian error propagation
% -> tedious... 'n probaly not worth it, but - here you are:
   
dx11 = dz(1,:);   dy11 = dx11;
dx12 = dz(2,:);   dy12 = dx12;
dx21 = dz(3,:);   dy21 = dx21;
dx22 = dz(4,:);   dy22 = dx22;

% ph11 = (x(4,:).*y(1,:)-x(2,:).*y(3,:))./det_x 
d_ph11__d_x11 =              0 + (-ph11./det_x).*  x(4,:);
d_ph11__d_x12 = -y(3,:)./det_x + (-ph11./det_x).*(-x(3,:));
d_ph11__d_x21 =              0 + (-ph11./det_x).*(-x(2,:));
d_ph11__d_x22 =  y(1,:)./det_x + (-ph11./det_x).*  x(1,:);

d_ph11__d_y11 =  x(4,:)./det_x + 0;
d_ph11__d_y12 =              0 + 0;
d_ph11__d_y21 = -x(2,:)./det_x + 0;
d_ph11__d_y22 =              0 + 0;

% ph12 = (x(4,:).*y(2,:) - x(2,:).*y(4,:)) ./ det_x
d_ph12__d_x11 =              0 + (-ph12./det_x).*  x(4,:);
d_ph12__d_x12 = -y(4,:)./det_x + (-ph12./det_x).*(-x(3,:));
d_ph12__d_x21 =              0 + (-ph12./det_x).*(-x(2,:));
d_ph12__d_x22 =  y(2,:)./det_x + (-ph12./det_x).*  x(1,:);

d_ph12__d_y11 =              0 + 0;
d_ph12__d_y12 =  x(4,:)./det_x + 0;
d_ph12__d_y21 =              0 + 0;
d_ph12__d_y22 = -x(2,:)./det_x + 0;

% ph21 = (x(1,:).*y(3,:) - x(3,:).*y(1,:)) / det_x
d_ph21__d_x11 =  y(3,:)./det_x + (-ph21./det_x).*  x(4,:);
d_ph21__d_x12 =              0 + (-ph21./det_x).*(-x(3,:));
d_ph21__d_x21 = -y(1,:)./det_x + (-ph21./det_x).*(-x(2,:));
d_ph21__d_x22 =              0 + (-ph21./det_x).*  x(1,:);

d_ph21__d_y11 = -x(3,:)./det_x + 0;
d_ph21__d_y12 =              0 + 0;
d_ph21__d_y21 =  x(1,:)./det_x + 0;
d_ph21__d_y22 =              0 + 0;

% ph22 = (x(1,:).*y(4,:) - x(3,:).*y(2,:))./ det_x
% --> ph11
dph11 = sqrt((d_ph11__d_x11.^2 + d_ph11__d_y11.^2).*dx11.^2 + ...
             (d_ph11__d_x12.^2 + d_ph11__d_y12.^2).*dx12.^2 + ...
             (d_ph11__d_x21.^2 + d_ph11__d_y21.^2).*dx21.^2 + ...
		     (d_ph11__d_x22.^2 + d_ph11__d_y22.^2).*dx22.^2);
 
dph12 = sqrt((d_ph12__d_x11.^2 + d_ph12__d_y11.^2).*dx11.^2 + ...
		     (d_ph12__d_x12.^2 + d_ph12__d_y12.^2).*dx12.^2 + ...
             (d_ph12__d_x21.^2 + d_ph12__d_y21.^2).*dx21.^2 + ...
		     (d_ph12__d_x22.^2 + d_ph12__d_y22.^2).*dx22.^2);
 
dph21 = sqrt((d_ph21__d_x11.^2 + d_ph21__d_y11.^2).*dx11.^2 + ...
		     (d_ph21__d_x12.^2 + d_ph21__d_y12.^2).*dx12.^2 + ...
             (d_ph21__d_x21.^2 + d_ph21__d_y21.^2).*dx21.^2 + ...
		     (d_ph21__d_x22.^2 + d_ph21__d_y22.^2).*dx22.^2);
 
dph22 = dph11;

% ph1 = 0.5 .* sqrt((ph11 - ph22).^2 + (ph12 + ph21).^2) 
d_ph1__d_ph11 =  0.5 .* 1./(2.*sqrt((ph11-ph22).^2 + (ph12 + ph21).^2))...
                     .* 2.*(ph11 - ph22);
d_ph1__d_ph12 =  0.5 .* 1./(2.*sqrt((ph11-ph22).^2 + (ph12 + ph21).^2))...
                     .* 2.*(ph12 + ph21);
d_ph1__d_ph21 =  0.5 .* 1./(2.*sqrt((ph11-ph22).^2 + (ph12 + ph21).^2))...
                     .* 2.*(ph12 + ph21);
d_ph1__d_ph22 = -0.5 .* 1./(2.*sqrt((ph11-ph22).^2 + (ph12 + ph21).^2))...
                     .* 2.*(ph11 - ph22);
	
% ph2 = 0.5 .* sqrt((ph11 + ph22).^2 + (ph12 - ph21).^2)
%  --> ph1
dph1 = sqrt(d_ph1__d_ph11.^2.*ph11.^2 + d_ph1__d_ph12.^2.*dph12.^2 + ...
            d_ph1__d_ph21.^2.*ph21.^2 + d_ph1__d_ph22.^2.*dph22.^2);  
 
dph2 = dph1; 

% phmax = ph1 + ph2
dphmax = sqrt(2).*dph1;

% phmin = ph2 - ph1
dphmin = dphmax;

% alpha = atan2(ph12 + ph21,ph11 - ph22) ./ 2
d_alpha__d_ph11 = .5 .* 1./(1 + ((ph12 + ph21)./(ph11 - ph22)).^2) .* ...
                                -(ph12 + ph21)./(ph11 - ph22) .^2;
d_alpha__d_ph12 = .5 .* 1./(1 + ((ph12 + ph21)./(ph11 - ph22)).^2) .* ...
                                            -1./(ph11 - ph22);
d_alpha__d_ph21 = .5 .* 1./(1 + ((ph12 + ph21)./(ph11 - ph22)).^2) .* ...
                                             1./(ph11 - ph22);
d_alpha__d_ph22 = -d_alpha__d_ph11;

dalpha = sqrt(d_alpha__d_ph11.^2.*dph11.^2 + d_alpha__d_ph12.^2.*dph12.^2 + ...
              d_alpha__d_ph21.^2.*dph21.^2 + d_alpha__d_ph22.^2.*dph22.^2); 

% beta = atan2(ph12-ph21,ph11+ph22)./2
% --> alpha
dbeta = dalpha;
%==========================================================================

ph_tens  = [ph11;ph12;ph21;ph22];
ph_svd   = [phmax;phmin;alpha;beta];
dph_tens = [dph11;dph12;dph21;dph22];
dph_svd  = [dphmax;dphmin;dalpha;dbeta];

return
